@@ -117,6 +117,21 @@ typedef enum _pdfio_predictor_e		// PNG predictor constants
   _PDFIO_PREDICTOR_PNG_AUTO = 15	// PNG "auto" predictor (currently mapped to Paeth)
 } _pdfio_predictor_t;
 
+typedef enum _pdfio_profile_e		// PDF profile constants
+{
+  _PDFIO_PROFILE_NONE = 0,		// Base PDF file
+  _PDFIO_PROFILE_PCLM,			// PCLm (PDF raster) file
+  _PDFIO_PROFILE_PDFA_1A,		// PDF/A-1a:2005
+  _PDFIO_PROFILE_PDFA_1B,		// PDF/A-1b:2005
+  _PDFIO_PROFILE_PDFA_2A,		// PDF/A-2a:2011
+  _PDFIO_PROFILE_PDFA_2B,		// PDF/A-2b:2011
+  _PDFIO_PROFILE_PDFA_2U,		// PDF/A-2u:2011
+  _PDFIO_PROFILE_PDFA_3A,		// PDF/A-3a:2012
+  _PDFIO_PROFILE_PDFA_3B,		// PDF/A-3b:2012
+  _PDFIO_PROFILE_PDFA_3U,		// PDF/A-3u:2012
+  _PDFIO_PROFILE_PDFA_4			// PDF/A-4:2020
+} _pdfio_profile_t;
+
 typedef ssize_t (*_pdfio_tconsume_cb_t)(void *data, size_t bytes);
 typedef ssize_t (*_pdfio_tpeek_cb_t)(void *data, void *buffer, size_t bytes);
 
@@ -239,6 +254,7 @@ struct _pdfio_file_s			// PDF file structure
   unsigned char	file_id[32];		// File identifier bytes
   struct lconv	*loc;			// Locale data
   char		*version;		// Version number
+  _pdfio_profile_t profile;		// PDF profile, if any
   pdfio_rect_t	media_box,		// Default MediaBox value
 		crop_box;		// Default CropBox value
   _pdfio_mode_t	mode;			// Read/write mode
@@ -268,10 +284,12 @@ struct _pdfio_file_s			// PDF file structure
   pdfio_obj_t	*info_obj;		// Information object
   pdfio_obj_t	*pages_obj;		// Root pages object
   pdfio_obj_t	*encrypt_obj;		// De/Encryption object/dictionary
-  pdfio_obj_t	*cp1252_obj,		// CP1252 font encoding object
+  pdfio_obj_t	*cgats001_obj,		// CGATS001 ICC profile object
+		*cp1252_obj,		// CP1252 font encoding object
 		*unicode_obj;		// Unicode font encoding object
   pdfio_array_t	*id_array;		// ID array
   bool		encrypt_metadata;	// Encrypt metadata?
+  pdfio_dict_t	*markinfo;		// MarkInfo dictionary, if any
 
   // Allocated data elements
   size_t	num_arrays,		// Number of arrays
